"use strict";

let cv, gl;

let program;

let vertexBuffer;

async function init() {
	cv = document.getElementById("cv");
	gl = cv.getContext("webgl");

	if (!gl) {
		window.alert("WebGL not supported");
	}

	console.log("compiling shaders...");

	console.log("compiling vertex shader...");
	let vertexShader = await fetchAndCompileShader(gl, gl.VERTEX_SHADER, "shaders/vertex.glsl");

	console.log("compiling fragment shader...");
	let fragmentShader = await fetchAndCompileShader(gl, gl.FRAGMENT_SHADER, "shaders/fragment.glsl");

	console.log("linking shader program...");
	program = linkShaderProgram(gl, vertexShader, fragmentShader);


	let triangleVertices = [
		 // X  // Y  // R // G // B
		 0.0,  0.5,  0.0, 1.0, 1.0,
		-0.5, -0.5,  1.0, 0.0, 1.0,
		 0.5, -0.5,  1.0, 1.0, 0.0
	];

	console.log("creating vertex buffer");

	vertexBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(triangleVertices), gl.STATIC_DRAW);

	// get location of vertPosition attribute
	let positionAttribLocation = gl.getAttribLocation(program, "vertPosition");

	gl.vertexAttribPointer(
		positionAttribLocation, // which attribute is read
		2, // number of values to read
		gl.FLOAT, // type of values
		gl.FALSE, // whether to normalize
		5 * Float32Array.BYTES_PER_ELEMENT, // size of individual vertex / distance between values to read
		0 // offset where to start reading
	);
	
	// enable attribute
	gl.enableVertexAttribArray(positionAttribLocation);

	let colorAttribLocation = gl.getAttribLocation(program, "vertColor");
	gl.vertexAttribPointer(
		colorAttribLocation,
		3,
		gl.FLOAT,
		gl.FALSE,
		5 * Float32Array.BYTES_PER_ELEMENT,
		2 * Float32Array.BYTES_PER_ELEMENT
	);
	gl.enableVertexAttribArray(colorAttribLocation);

	// unbind buffer
	gl.bindBuffer(gl.ARRAY_BUFFER, null);

	// set clear colour
	gl.clearColor(0.1, 0.1, 0.1, 1.0);

	// start drawing
	requestAnimationFrame(draw);
}

async function draw() {
	gl.clear(gl.COLOR_BUFFER_BIT);

	gl.useProgram(program);

	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);

	gl.drawArrays(gl.TRIANGLES, 0, 3);

	requestAnimationFrame(draw);
}

window.addEventListener("DOMContentLoaded", init);